use darling::FromMeta;
use proc_macro::TokenStream;
use proc_macro2::Span;
use proc_macro_crate::{crate_name, FoundCrate};
use quote::quote;
use syn::{parse_macro_input, AttributeArgs, Ident, ItemFn, Signature};

#[derive(FromMeta)]
struct AttrArgs {
    #[darling(default)]
    description: String,
}

#[proc_macro_attribute]
pub fn builtin(attr: TokenStream, item: TokenStream) -> TokenStream {
    let attrs = parse_macro_input!(attr as AttributeArgs);
    let ItemFn {
        vis, sig, block, ..
    } = parse_macro_input!(item as ItemFn);
    let Signature {
        ident,
        fn_token,
        inputs,
        ..
    } = sig;

    let name = ident.to_string();
    let AttrArgs { description } = match AttrArgs::from_list(&attrs) {
        Ok(args) => args,
        Err(err) => return err.write_errors().into(),
    };

    let builtin = match crate_name("oyster_runtime").expect("oyster_runtime missing") {
        FoundCrate::Itself => quote! { crate::builtins::Builtin },
        FoundCrate::Name(name) => {
            let ident = Ident::new(&name, Span::call_site());
            quote! { #ident::builtins::Builtin }
        }
    };

    TokenStream::from(quote! {
        #[allow(non_upper_case_globals)]
        #vis static #ident: #builtin = {
            #fn_token #ident(#inputs) #block

            #builtin {
                name: #name,
                description: #description,
                fun: #ident,
            }
        };
    })
}
