use crate::{NodeKind, ParseError, ParseEvent, Parser};

/// A concrete, loss-less representation of the parsed program.
#[derive(Debug)]
pub enum ParseTree {
    Tree {
        kind: NodeKind,
        children: Vec<ParseTree>,
    },
    Leaf {
        kind: NodeKind,
        len: usize,
    },
    Error {
        kind: ParseError,
        len: usize,
    },
}

impl<'a> From<&'a str> for ParseTree {
    fn from(source: &str) -> Self {
        let mut parser = Parser::new(source);

        ParseTree::Tree {
            kind: NodeKind::Program,
            children: build_tree(&mut parser),
        }
    }
}

fn build_tree(parser: &mut Parser) -> Vec<ParseTree> {
    let mut children = vec![];

    while let Some(ev) = parser.next() {
        match ev {
            ParseEvent::EndNode => break,
            ParseEvent::StartNode(kind) => children.push(ParseTree::Tree {
                kind,
                children: build_tree(parser),
            }),
            ParseEvent::Error(kind, len) => children.push(ParseTree::Error { kind, len }),
            ParseEvent::NewLeaf(kind, len) => children.push(ParseTree::Leaf { kind, len }),
        }
    }

    children
}
